//---------------------------------------------------------------------------
// TDcjRegistryDlg Component
//
// Copyright  1998, Michael Trier.  All Rights Reserved.
//---------------------------------------------------------------------------
// This component and any associated property editors and/or component editors
// can be freely used and distributed, provided all source code files are not
// modified in any way and there is no charge for it other than nominal shipping
// and handling fees.  Please contact me directly if you desire to modify the
// terms of this agreement.
//---------------------------------------------------------------------------
// Please note:  This source code is provided as a teaching tool. If you
// decide that you want to use whole routines or modules directly in an
// application that you are creating, please give credit where it is due.
// Given the above, keep in mind that these applications are copyright
// property of Michael Trier.
//---------------------------------------------------------------------------
// Email: mtrier@pobox.com
// Website: www.dcjournal.com
//---------------------------------------------------------------------------
// Date last modified: $Date: 1998/11/14 19:34:02 $
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// TDcjRegistryDlg $Revision: 0.53 $
//---------------------------------------------------------------------------
// Revision History:
// $Log: DcjRegDlg.cpp,v $
// Revision 0.53  1998/11/14 19:34:02  michael
// - Added pragma link so DcjCustomDlg doesn't need to be included in the Package
// - Added Sorted property to allow sorting of TreeView info
// - Removed some testing code
// - Fixed a problem with the Text label where it wasn't word wrapping properly
// - Added a component editor to allow previewing during design time
// - Added hourglass during initial load.
//
// Revision 0.52  1998/11/14 15:27:29  michael
// - Seperated TDcjRootKey class to its own unit
// - Included new DcjRootKey header
// - Changed TNodePath class to TDcjNodePath and altered private member variables
//
// Revision 0.51  1998/11/14 14:53:50  michael
// - Added ShowNode property
// - Fixed some bugs with empty nodepaths
// - Add code to support rkNone, where it shows all RootKeys
//
// Revision 0.50  1998/11/12 14:40:13  michael
// - Initial revision
//
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#include <memory>
#pragma hdrstop

#include "DcjRegDlg.h"
#pragma link "DcjCustomDlg"
#pragma link "DcjRootKey"
#ifndef DCJ_BCB_1
  #pragma resource "*.res"
  #pragma package(smart_init)
#endif
//---------------------------------------------------------------------------
static inline TDcjRegistryDlg *ValidCtrCheck()
{
  return new TDcjRegistryDlg(NULL);
}
//---------------------------------------------------------------------------
// TDcjNodePath Implementation
//---------------------------------------------------------------------------
__fastcall TDcjNodePath::TDcjNodePath(AnsiString sNodePath, TDcjRootKey aRootKey)
  : TObject()
{
    // assign the NodePath string to the private member variable
    FNodePath = sNodePath;
    FRootKey = aRootKey;
}

//---------------------------------------------------------------------------
// TDcjRegistryForm Implementation
//---------------------------------------------------------------------------
__fastcall TDcjRegistryForm::TDcjRegistryForm(Classes::TComponent* AOwner,
    int Dummy)
  : TDcjDialogForm(AOwner, Dummy)
{
  iNodeLevel = 0;
  iCurrentLevel = 0;
  pCurrentNode = NULL;
  FLoadOnDemand = true;
  FShowRoot = false;
}
//---------------------------------------------------------------------------
void __fastcall TDcjRegistryForm::LoadRegistryInfo(TDcjRootKey aRootKey,
    AnsiString sCurrentPath)
{
  // fill the tree view with registry information

  // store the screen cursor, change the cursor to an hourglass
  TCursor oldcursor = Screen->Cursor;
  Screen->Cursor = crHourGlass;

  // turn off updating on the listview and clear it as well
  tvwRegistry->Items->BeginUpdate();
  tvwRegistry->Items->Clear();

  // initialize the node tracking
  iNodeLevel = 0;
  iCurrentLevel = 0;

  // check if the root key is rkNone. if it is, cycle through all root keys.
  if( aRootKey.RootKey == rkNone ) {
    // OnDemand mode required when loading all rootkeys
    LoadOnDemand = true;
    for( int i = 1; i < 5; i++ ) {
      // cycle through root keys
      pCurrentNode = NULL;
      aRootKey.RootKey = i;
      AddKeyToTree(aRootKey, "", aRootKey.RootKeyAsHKEYString);
      // add placeholder keys underneath the rootkeys
      iNodeLevel++;
      AddKeyToTree(aRootKey, "placeholder", "placeholder");
      // initialize the node tracking
      iNodeLevel = 0;
      iCurrentLevel = 0;
    }
  }
  else {
    // check if we need to add the root key
    if( ShowRoot ) {
      // add the rootkey
      AddKeyToTree(aRootKey, "", aRootKey.RootKeyAsHKEYString);
      iNodeLevel++;
    }
    // load the registry key specified
    LoadRegistryKey(aRootKey, sCurrentPath);
  }
  // if it's sorted we need to AlphaSort at this point
  if( tvwRegistry->SortType != stNone )
    tvwRegistry->AlphaSort();

  // update the treeview
  tvwRegistry->Items->EndUpdate();

  // restore the cursor
  Screen->Cursor = oldcursor;
}
//---------------------------------------------------------------------------
void __fastcall TDcjRegistryForm::LoadRegistryKey(TDcjRootKey aRootKey,
    AnsiString sCurrentPath)
{
  // load the registry starting from the root key and current key specified

 	// create a registy object
  TRegistry *registry = new TRegistry();
  try {
    // set the rootkey
    registry->RootKey = aRootKey.RootKeyAsHKEY;
    // set our subkey
    if (registry->OpenKey(sCurrentPath, false)) {
      // retrieve the settings from the registry
      try {
        // check if the node path is in a subkey
        int lastdelim = sCurrentPath.LastDelimiter("\\");
        AnsiString regkey;
        if( lastdelim ) {
          // strip the key from the path
          regkey = sCurrentPath.SubString(lastdelim + 1,
              sCurrentPath.Length() - lastdelim);
        }

        // add the key name to the tree
        if( !sCurrentPath.IsEmpty() )
          AddKeyToTree(aRootKey, sCurrentPath, regkey);
        // check if the key has subkeys
        if( registry->HasSubKeys() ) {
          // if we are in on demand mode, then only load subkeys
          if( LoadOnDemand ) {
            // load the sub keys
            TStringList *keys = new TStringList();
            registry->GetKeyNames(keys);
            // close the current key
            registry->CloseKey();
            iNodeLevel++;
            for( int i = 0; i < keys->Count; i++ ) {
              // add the key name to the tree
              AddKeyToTree(aRootKey, sCurrentPath + "\\" + keys->Strings[i],
                  keys->Strings[i]);
              // Open the key, and find out if there are subkeys
              if( registry->OpenKey(sCurrentPath + "\\" + keys->Strings[i],
                  false) ) {
                // check if it has subkeys
                if( registry->HasSubKeys() ) {
                  iNodeLevel++;
                  // add a dumby node as a placeholder
                  AddKeyToTree(aRootKey, sCurrentPath + "\\" + keys->Strings[i] + "\\" +
                      "placeholder", "placeholder");
                  iNodeLevel--;
                }
                registry->CloseKey();
              }
            }
            delete keys;
            keys = NULL;
          }
          else
            // load the whole tree
            RecurseRegistry(aRootKey, sCurrentPath);
        }
      }
      catch(...) {
        Application->MessageBox("Unable to locate registry key", "Registry Error",
            MB_OK | MB_ICONEXCLAMATION);
      }
    }
  }
  catch(...) {
    if( registry ) {
      delete registry;
      registry = NULL;
    }
  }
  delete registry;
  registry = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TDcjRegistryForm::RecurseRegistry(TDcjRootKey aRootKey,
    AnsiString sCurrentPath)
{
  iNodeLevel++;
  // recurse the registry keys from the current key
  TStringList *keys = new TStringList();
  AnsiString newpath;

 	// create a registy object
  TRegistry *registry = new TRegistry();
  // open to the aRootKey
  registry->RootKey = aRootKey.RootKeyAsHKEY;
  // open the current key
  if( registry->OpenKey(sCurrentPath, false) ) {
    // find out if there are subkeys
    registry->GetKeyNames(keys);
    for( int i = 0; i < keys->Count; i++ ) {
      // add the key name to the tree
      AddKeyToTree(aRootKey, sCurrentPath + "\\" + keys->Strings[i],
          keys->Strings[i]);
      // close the current key
      registry->CloseKey();
      // open the key, and find out if it has subkeys
      newpath = sCurrentPath + "\\" + keys->Strings[i];
      if( registry->OpenKey(newpath, false) ) {
        // find out if it has subkeys
        if( registry->HasSubKeys() )
          // process subkeys
          RecurseRegistry(aRootKey, newpath);
      }
    }
  }
  iNodeLevel--;
  delete registry;
  registry = NULL;
  delete keys;
  keys = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TDcjRegistryForm::AddKeyToTree(TDcjRootKey aRootKey,
    AnsiString sNodePath, AnsiString sKey)
{
  // add the key to the treeview control

  TDcjNodePath *object = new TDcjNodePath(sNodePath, aRootKey);

  // check if the node level is greater than the current level
  if( iNodeLevel > iCurrentLevel )
    // indent the tree node
    pCurrentNode = tvwRegistry->Items->AddChildObject(pCurrentNode, sKey, object);
  else {
    // check if the node level is less than the current level
    if( iNodeLevel < iCurrentLevel ) {
      // get the parents by the same number of the difference
      for( int i = 0; i < iCurrentLevel - iNodeLevel; i++ )
        pCurrentNode = pCurrentNode->Parent;
      // add the sibling
      pCurrentNode = tvwRegistry->Items->AddObject(pCurrentNode, sKey, object);
    }
    else
      // add a sibling
      pCurrentNode = tvwRegistry->Items->AddObject(pCurrentNode, sKey, object);
  }
  // make them the same again
  iCurrentLevel = iNodeLevel;
}
//---------------------------------------------------------------------------
void __fastcall TDcjRegistryForm::cmdOKClick(TObject *Sender)
{
  // return the node path to the user
  TDcjNodePath *nodepath = (TDcjNodePath *)tvwRegistry->Selected->Data;
  NodePath = nodepath->NodePath;
  RootKey = nodepath->RootKey;
}
//---------------------------------------------------------------------------
void __fastcall TDcjRegistryForm::cmdCancelClick(TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------
void __fastcall TDcjRegistryForm::tvwRegistryExpanding(TObject *Sender,
      TTreeNode *Node, bool &AllowExpansion)
{
  // check if we are in on demand mode
  if( LoadOnDemand ) {
    // find out if there ia only single subitem.  if there is, it is probably
    // our placeholder, otherwise, the tree has already been expanded.
    if( Node->Count < 2 ) {
      // store the screen cursor, change the cursor to an hourglass
      TCursor oldcursor = Screen->Cursor;
      Screen->Cursor = crHourGlass;
      // delete the children nodes
      Node->DeleteChildren();
      // find out if there are children available
      TDcjNodePath *np = (TDcjNodePath *)Node->Data;
      AnsiString nodepath = np->NodePath;
      TDcjRootKey rootkey = np->RootKey;
      TRegistry *registry = new TRegistry;
      // open to the correct root key
      registry->RootKey = rootkey.RootKeyAsHKEY;
      TStringList *keys = new TStringList();
      if( registry->OpenKey(nodepath, false) ) {
        if( registry->HasSubKeys() ) {
          // add them to the node
          registry->GetKeyNames(keys);
          registry->CloseKey();
          // set the nodelevel appropriately to indicate that we are expanding
          // in by a single level.
          iNodeLevel = iCurrentLevel + 1;
          // set the current node to the node that we are expanding.
          pCurrentNode = Node;
          // cycle through the keys to add
          for( int i = 0; i < keys->Count; i++ ) {
            // add the key name to the tree
            AddKeyToTree(rootkey, nodepath + "\\" + keys->Strings[i],
                keys->Strings[i]);
            // Open the key, and find out if there are subkeys
            if( registry->OpenKey(nodepath + "\\" + keys->Strings[i],
                false) ) {
              // check if it has subkeys
              if( registry->HasSubKeys() ) {
                iNodeLevel++;
                // add a dumby node as a placeholder
                AddKeyToTree(rootkey, nodepath + "\\" + keys->Strings[i] + "\\" +
                    "placeholder", "placeholder");
                iNodeLevel--;
              }
              registry->CloseKey();
            }
          }
        }
        else
          // don't allow expansion
          AllowExpansion = false;
      }
      delete keys;
      keys = NULL;
      delete registry;
      registry = NULL;
      // if it's sorted, AlphaSort the expanded node at this point
      if( tvwRegistry->SortType != stNone )
        Node->AlphaSort();
      // restore the cursor
      Screen->Cursor = oldcursor;
    }
  }
}
//---------------------------------------------------------------------------
void __fastcall TDcjRegistryForm::tvwRegistryDeletion(TObject *Sender,
      TTreeNode *Node)
{
  // free up the memory associated with the node
  TDcjNodePath *nodepath = (TDcjNodePath *)Node->Data;
  delete nodepath;
  nodepath = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TDcjRegistryForm::tvwRegistryClick(TObject *Sender)
{
  // enable the OK command button
  if( cmdOK->Enabled == false )
    cmdOK->Enabled = true;
}
//---------------------------------------------------------------------------
// TDcjRegistryDlg Implementation
//---------------------------------------------------------------------------
__fastcall TDcjRegistryDlg::TDcjRegistryDlg(TComponent* Owner)
  : TDcjCustomDlg(Owner)
{
  FNodePath = "";
  FLoadOnDemand = true;
  FShowRoot = false;
  FSorted = false;
}
//---------------------------------------------------------------------------
__fastcall TDcjRegistryDlg::~TDcjRegistryDlg(void)
{
}
//---------------------------------------------------------------------------
TDcjRootKeyEnum __fastcall TDcjRegistryDlg::GetRootKey(void)
{
  // return the RootKey as an enum
  return FRootKey.RootKey;
}
//---------------------------------------------------------------------------
void __fastcall TDcjRegistryDlg::SetRootKey(TDcjRootKeyEnum Value)
{
  // set the RootKey from the enum
  if( FRootKey.RootKey != Value )
    FRootKey.RootKey = Value;
}
//---------------------------------------------------------------------------
void __fastcall TDcjRegistryDlg::Assign(TPersistent *Source)
{
  // assign the source object to the destination
  if( Source->InheritsFrom(__classid(TDcjRegistryDlg)) ) {
    // cast the TPersistant object to a TDcjRegistryDlg object
    TDcjRegistryDlg *source = dynamic_cast<TDcjRegistryDlg *>(Source);
    // assign the inherited properties
    TDcjCustomDlg::Assign(source);
    // assign the properties
    RootKeyObject = source->RootKeyObject;
    NodePath = source->NodePath;
    ShowRoot = source->ShowRoot;
  }
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDcjRegistryDlg::PreviewResults(void)
{
  // return the results of the users selection
  AnsiString result = "RootKey: " + RootKeyObject.RootKeyAsString + "\r\n";
  result = result + "NodePath: " + NodePath;
  return result;
}
//---------------------------------------------------------------------------
bool __fastcall TDcjRegistryDlg::Execute(void)
{
  // create the message dialog and return the result
  bool result = false;

  TDcjRegistryForm *dialog = CreateDcjDialog();
  try {
    // display the dialog and return the settings
    ModalResult = dialog->ShowModal();
    if( ModalResult == mrOk ) {
      NodePath = dialog->NodePath;
      RootKey = dialog->RootKey.RootKey;
    }
    result = true;
  }
  catch(...) {
    delete dialog;
    dialog = NULL;
  }
  // free up the memory
  delete dialog;
  dialog = NULL;

  return result;
}
//---------------------------------------------------------------------------
TDcjRegistryForm* __fastcall TDcjRegistryDlg::CreateDcjDialog(void)
{
  // creates the registry dialog and retrieves the users selection
  TDcjRegistryForm *RegForm = new TDcjRegistryForm(Application, 1);
  // create and set the form objects
  CreateFormObjects(RegForm);
  // alter properties appropriately
  RegForm->Font->Assign(Font);
  RegForm->HelpContext = HelpContext;
  RegForm->Caption = Caption;
  RegForm->lblText->Caption = Text;
  // set the on demand setting
  RegForm->LoadOnDemand = LoadOnDemand;
  // set the show root setting
  RegForm->ShowRoot = ShowRoot;
  // set the sorted property
  if( Sorted )
    RegForm->tvwRegistry->SortType = stBoth;

  // set the rootkey and nodepath properties
  RegForm->RootKey = RootKeyObject;
  RegForm->NodePath = NodePath;

  // assign event functions
  RegForm->cmdHelp->OnClick = RegForm->HelpButtonClick;

/*
  // position the dialog in the center of the screen
  RegForm->Left = (div(Screen->Width, 2).quot) - (div(RegForm->Width, 2).quot);
  RegForm->Top = (div(Screen->Height, 2).quot) - (div(RegForm->Height, 2).quot);
*/

  // fill the tree view with the registry information
  RegForm->LoadRegistryInfo(RootKeyObject, NodePath);

  return RegForm;
}
//---------------------------------------------------------------------------
void __fastcall TDcjRegistryDlg::CreateFormObjects(TDcjRegistryForm *RegForm)
{
  // setting property values for RegForm, an object of theTRegForm class
  RegForm->Left = 219;
  RegForm->Top = 112;
  RegForm->Width = 326;
  RegForm->Height = 402;
  RegForm->Caption = "Registry Browser";
  RegForm->PixelsPerInch = 96;

  // setting property values for lblText, an object of theTLabel class
  /* TLabel * */RegForm->lblText = new TLabel(RegForm);
  RegForm->lblText->Parent = RegForm;
  RegForm->lblText->Left = 8;
  RegForm->lblText->Top = 8;
  RegForm->lblText->Width = 297;
  RegForm->lblText->Height = 39;
  RegForm->lblText->AutoSize = false;
  RegForm->lblText->Caption = "";
  RegForm->lblText->WordWrap = true;

  // setting property values for tvwRegistry, an object of theTTreeView class
  /* TTreeView * */RegForm->tvwRegistry = new TTreeView(RegForm);
  RegForm->tvwRegistry->Parent = RegForm;
  RegForm->tvwRegistry->Left = 8;
  RegForm->tvwRegistry->Top = 48;
  RegForm->tvwRegistry->Width = 297;
  RegForm->tvwRegistry->Height = 289;
  RegForm->tvwRegistry->ReadOnly = True;
  RegForm->tvwRegistry->Indent = 19;
  RegForm->tvwRegistry->OnExpanding = RegForm->tvwRegistryExpanding;
  RegForm->tvwRegistry->OnDeletion = RegForm->tvwRegistryDeletion;
  RegForm->tvwRegistry->TabOrder = 0;
  RegForm->tvwRegistry->OnClick = RegForm->tvwRegistryClick;

  // setting property values for cmdCancel, an object of theTButton class
  /* TButton * */RegForm->cmdCancel = new TButton(RegForm);
  RegForm->cmdCancel->Parent = RegForm;
  RegForm->cmdCancel->Left = 144;
  RegForm->cmdCancel->Top = 344;
  RegForm->cmdCancel->Width = 75;
  RegForm->cmdCancel->Height = 25;
  RegForm->cmdCancel->Caption = "&Cancel";
  RegForm->cmdCancel->ModalResult = 2;
  RegForm->cmdCancel->TabOrder = 1;
  RegForm->cmdCancel->OnClick = RegForm->cmdCancelClick;

  // setting property values for cmdOK, an object of theTButton class
  /* TButton * */RegForm->cmdOK = new TButton(RegForm);
  RegForm->cmdOK->Parent = RegForm;
  RegForm->cmdOK->Left = 58;
  RegForm->cmdOK->Top = 344;
  RegForm->cmdOK->Width = 75;
  RegForm->cmdOK->Height = 25;
  RegForm->cmdOK->Caption = "&OK";
  RegForm->cmdOK->ModalResult = 1;
  RegForm->cmdOK->TabOrder = 2;
  RegForm->cmdOK->Enabled = false;
  RegForm->cmdOK->OnClick = RegForm->cmdOKClick;

  // setting property values for cmdHelp, an object of theTButton class
  /* TButton * */RegForm->cmdHelp = new TButton(RegForm);
  RegForm->cmdHelp->Parent = RegForm;
  RegForm->cmdHelp->Left = 230;
  RegForm->cmdHelp->Top = 344;
  RegForm->cmdHelp->Width = 75;
  RegForm->cmdHelp->Height = 25;
  RegForm->cmdHelp->Caption = "&Help";
  RegForm->cmdHelp->TabOrder = 3;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDcjRegistryDlg::GetVersion(void)
{
  // return the component version number as a string
  return "$Revision: 0.53 $";
}
//---------------------------------------------------------------------------
namespace Dcjregdlg
{
  void __fastcall PACKAGE Register()
  {
    // register component
    TComponentClass classes[1] = {__classid(TDcjRegistryDlg)};
    RegisterComponents("Dcj", classes, 0);
  }
}
//---------------------------------------------------------------------------

